<?php
/**
 * @file
 * jQuad's Form Functions
 */

/**
 * Implementation of hook_form_alter().
 */
function jquad_form_alter(&$form, $form_state, $form_id) {
  // Only alter forms that are allowed for validation
  $forms = jquad_allowed_forms();
  if (array_key_exists ($form_id, $forms)) {
    // Ensure the that jQuad should alter this form
    if ((arg(0) == 'ajax' && arg(1) == 'jquad')) {
      $support = jquad_support();
      // Set form path
      $path = $forms[$form_id];
      // Ensure path is secure if supported
      if ($support['securepages']) {
        $secure = (bool)securepages_match($path);
        $path = securepages_url($path, array('absolute' => TRUE, 'secure' => $secure));
      }
      else {
        $path = url($path, array('absolute' => TRUE));
      }
      $form['md5'] = array(
        '#type' => 'hidden',
        '#value' => (variable_get('jquad_md5', TRUE) && empty($conflicts)) ? 1 : 0,
      );
      // Change the form's post URL. More for aesthetics, but could also be used for fallback (in theory).
      $form['#action'] = $path;
      // If possible, use the supported module's jQuad validation rewrite. Needed to provide proper feedback using JSON.
      foreach ($support as $module => $data) {
        if (is_array($data) && isset($data['include'])) {
          if (is_file($data['include'])) {
            require_once $data['include'];
          }
        }
        if (is_array($data) && isset($data['validate'])) {
          // Only run use validation function if it exists
          $function = 'jquad_' . $data['validate'];
          if (function_exists($function)) {
            // Replace the original validation function with the jQuad equivilant.
            $key = array_search($data['validate'], (array) $form['#validate']);
            if ($key !== FALSE) {
              $form['#validate'][$key] = 'jquad_' . $data['validate'];
            }
            // Append the validation function to run at the end
            else {
              $form['#validate'][] = 'jquad_' . $data['validate'];
            }
          }
          else {
            form_set_error('', 'Cannot find function: "' . $function . '".');
          }
        }
      }
      // Append the default jQuad validation function to return form errors in JSON. Should always be executed after everything else.
      $form['#validate'][] = 'jquad_validate_form';
      // Append the default jQuad submit function to return JSON data after the form has validated. Should always be executed after everything else.
      $form['#submit'][] = 'jquad_submit_form';
    }
  }
}

function jquad_validate_form($form, &$form_state) {
  $form_values = $form_state['clicked_button']['#post'];
  // Revert messages to previous state so any messages created by the submit handlers can be shown on main site after redirect.
  $_SESSION['messages'] = $_SESSION['site_messages'];
  // If form has errors, return errors immeadiately so user can correct it.
  $errors = form_get_errors();
  if (!empty($errors)) {
    jquad_return_form(array(
      'status' => FALSE,
      'messages' => $errors,
    ), $form_values);
  }
}

function jquad_submit_form($form, &$form_state) {
  $json = array(
    'status' => TRUE,
    'messages' => array(),
  );
  if (!empty($form_state['redirect']) && !(bool) variable_get('jquad_redirect', TRUE)) {
    $json['redirect'] = url($form_state['redirect'], array('absolute' => TRUE));
  }
  jquad_return_form($json, $form_state['clicked_button']['#post']);
}


function jquad_get_form($form_id) {
  // Process supported module's forms
  if (isset($_GET['token'])) {
    $support = jquad_support();
    foreach ($support as $module => $data) {
      if ($module == $form_id) {
        if (is_array($data) && isset($data['include']) && is_file($data['include'])) {
          require_once $data['include'];
        }
        $function = $module . '_jquad_get_form';
        if (function_exists($function)) {
          return $function($_GET['token']);
          exit();
        }
      }
    }
  }
  global $user;
  $json = array(
    'status' => FALSE,
  );
  // Otherwise process normal forms
  if (array_key_exists($form_id, jquad_allowed_forms())) {
    switch ($form_id) {
      case 'user_login' || 'user_register' || 'user_pass' || 'logout':
        module_load_include('inc', 'user', 'user.pages');
        break;
    }
    $json['status'] = TRUE;
    if ($form_id == 'logout') {
      $json['form'] = '<form>' . t('Do you want to logout of your current session?') . '<input type="hidden" name="form_id" value="logout" /><input type="submit" value="Logout" /></form>';
    }
    else {
      $json['form'] = drupal_get_form($form_id);
    }
  }
  else {
    $json['status'] = FALSE;
    $json['html'] = t('An error occurred while retrieving the form: "' . $form_id . '".');
    switch ($form_id) {
      case 'user_login':
        if ($user->uid) {
          $json['html'] = t('You are already logged in.');
        }
        break;
    }
  }
  drupal_json($json);
  exit();
}

function jquad_process_form($form_id) {
  if (array_key_exists($form_id, jquad_allowed_forms())) {
    $form_values = $_POST;
    if ($form_id == 'logout') {
      watchdog('user', 'Session closed for %name.', array('%name' => $user->name));
      // Destroy the current session:
      session_destroy();
      // Only variables can be passed by reference workaround.
      $null = NULL;
      user_module_invoke('logout', $null, $user);
      // Load the anonymous user
      $user = drupal_anonymous_user();
      jquad_return_form(array(
        'redirect' => url('<front>', array('absolute' => TRUE)),
      ), $form_values);
    }
    else {
      // When forms validate, they can throw form_set_error which in turn calls drupal_set_messages.
      // Ensure we save current site's message state so we don't generate duplicates messages from form errors to the main site
      $_SESSION['site_messages'] = drupal_get_messages();
      // Process form for validation and submission
      $form_state = array('values' => $form_values, 'submitted' => TRUE);
      drupal_execute($form_id, $form_state);
    }
  }
  else {
    jquad_return_form(array(
      'status' => FALSE,
      'messages' => array('jQuad cannot process the following invalid form: "' . $form_id . '".'),
    ), $form_values);
  }
}

function jquad_return_form($json = array(), $form_values) {
  // Load default JSON return data
  $json = array_merge(array(
    'status' => TRUE,
    'messages' => array(),
  ), $json);
  // Determine which type of response to send back
  if (isset($form_values['windowname'])) {
    print '<script type="text/javascript">window.name=\'' . jquad_to_js($json) . '\';</script>';
  }
  else {
    print drupal_json($json);
  }
  exit();
}
