<?php
/**
 * @file
 * jQuad's OpenID Support File
 */

/**
 * Login form _validate hook
 */
function jquad_openid_login_validate($form, &$form_state) {
  $return_to = $form_state['values']['openid.return_to'];
  if (empty($return_to)) {
    $return_to = url('', array('absolute' => TRUE));
  }
  jquad_openid_begin($form_state['values']['openid_identifier'], $return_to, $form_state['values']);
}

/**
 * The initial step of OpenID authentication responsible for the following:
 *  - Perform discovery on the claimed OpenID.
 *  - If possible, create an association with the Provider's endpoint.
 *  - Create the authentication request.
 *  - Perform the appropriate redirect.
 *
 * @param $claimed_id The OpenID to authenticate
 * @param $return_to The endpoint to return to from the OpenID Provider
 */
function jquad_openid_begin($claimed_id, $return_to = '', $form_values = array()) {
  module_load_include('module', 'openid');
  module_load_include('inc', 'openid');

  $claimed_id = _openid_normalize($claimed_id);

  $services = openid_discovery($claimed_id);
  if (count($services) == 0) {
    form_set_error('openid_identifier', t('Sorry, that is not a valid OpenID. Please ensure you have spelled your ID correctly.'));
  }
  // If form has errors, return errors immeadiately so user can correct it.
  $errors = form_get_errors();
  if (!empty($errors)) {
    jquad_return_form(array(
      'status' => FALSE,
      'messages' => $errors,
    ), $form_values);
  }
  // Store discovered information in the users' session so we don't have to rediscover.
  $_SESSION['openid']['service'] = $services[0];
  // Store the claimed id
  $_SESSION['openid']['claimed_id'] = $claimed_id;
  // Store the login form values so we can pass them to
  // user_exteral_login later.
  $_SESSION['openid']['user_login_values'] = $form_values;

  $op_endpoint = $services[0]['uri'];
  // If bcmath is present, then create an association
  $assoc_handle = '';
  if (function_exists('bcadd')) {
    $assoc_handle = openid_association($op_endpoint);
  }
  // Now that there is an association created, move on
  // to request authentication from the IdP
  // First check for LocalID. If not found, check for Delegate. Fall
  // back to $claimed_id if neither is found.
  if (!empty($services[0]['localid'])) {
    $identity = $services[0]['localid'];
  }
  elseif (!empty($services[0]['delegate'])) {
    $identity = $services[0]['delegate'];
  }
  else {
    $identity = $claimed_id;
  }
  if (isset($services[0]['types']) && is_array($services[0]['types']) && in_array(OPENID_NS_2_0 .'/server', $services[0]['types'])) {
    $claimed_id = $identity = 'http://specs.openid.net/auth/2.0/identifier_select';
  }
  $authn_request = openid_authentication_request($claimed_id, $identity, $return_to, $assoc_handle, $services[0]['version']);
  // Return OpenID JSON data
  $json = array(
    'status' => TRUE,
    'messages' => array(),
  );
  // Generate a random token to save data on server temporarily
  $token = 'jquad-openid-' . md5(mt_rand());
  $_SESSION[$token] = array(
    'url' => $op_endpoint,
    'message' => $authn_request,
    'protocol' => $services[0]['version'],
  );
  $json['callback'] = 'openId';
  $json['arguments'] = $token;
  jquad_return_form($json, $form_values);
}

function openid_jquad_get_form($token) {
  if (isset($_SESSION[$token])) {
    module_load_include('inc', 'openid');
    $data = $_SESSION[$token];
    if ($data['protocol'] == 2) {
      print openid_redirect($data['url'], $data['message']);
    }
    else {
      print openid_redirect_http($data['url'], $data['message']);
    }
  }
  else {
    print t('There was an error processing your OpenID.');
  }
  exit();
}

/**
 * Performs an HTTP redirect (for the 1.x protocol).
 */
function jquad_openid_redirect_http($url, $message) {
  $query = array();
  foreach ($message as $key => $val) {
    $query[] = $key .'='. urlencode($val);
  }
  $sep = (strpos($url, '?') === FALSE) ? '?' : '&';
  $url = $url . $sep . implode('&', $query);
  $output = '<html><head><title>'. t('OpenID redirect') . '</title>';
  $output .= '<meta http-equiv="Refresh" content="0; url=' . $url . '" />';
  $output .= '</head>\n<body>';
  $output .= '<p>Open ID redirect, if page does not refresh please click <a href="' . $url . '">this link</a>.</p>';
  $output .= "</body></html>\n";
  return $output;
}

/**
 * Creates a js auto-submit redirect for (for the 2.x protocol)
 */
function jquad_openid_redirect($url, $message) {
  return drupal_get_form('openid_redirect_form', $url, $message);
}


